// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2017 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//
// This ROM stores the address offset for PLL that needs reconfig
//
module mr_rom_pll_dprioaddr_bitmask #(
    parameter ROM_SIZE        = 18,
    parameter TOTAL_ROM_DEPTH = 16,
    parameter ADDR_WIDTH      = 4			      
) (
    input  wire                  clock,
    input  wire [ADDR_WIDTH-1:0] addr_ptr,
    output wire [ROM_SIZE-1:0]   rdata_out
);

reg  [ROM_SIZE-1:0]   ROM [0:TOTAL_ROM_DEPTH-1];
wire [ROM_SIZE-1:0]   DATAA = {ROM_SIZE{1'b0}};
wire [ADDR_WIDTH-1:0] RADDR;
   
initial begin
           // ADDR_OFST 
    ROM[0] <= {8'hFF, 10'h104}; // M Counter High
    ROM[1] <= {8'hFF, 10'h107}; // M Counter Low
    ROM[2] <= {8'hFF, 10'h100}; // N Counter High
    ROM[3] <= {8'hFF, 10'h102}; // N Counter Low
    ROM[4] <= {8'h01, 10'h101}; // N Counter Bypass Enable
    ROM[5] <= {8'hFF, 10'h11B}; // C0 Counter High
    ROM[6] <= {8'hFF, 10'h11E}; // C0 Counter Low
    ROM[7] <= {8'hFF, 10'h11F}; // C1 Counter High
    ROM[8] <= {8'hFF, 10'h122}; // C1 Counter Low
    ROM[9] <= {8'h80, 10'h121}; // C1 Even Duty Enable
    ROM[10] <= {8'h70, 10'h101}; // Charge Pump Current [2:0]
    ROM[11] <= {8'hE0, 10'h10D}; // Charge Pump Current [5:3]
    ROM[12] <= {8'h78, 10'h10A}; // Bandwidth
    ROM[13] <= {8'h40, 10'h149}; // Request Calibration
    ROM[14] <= {8'hFF, 10'h14A}; // Enable Calibration Interface
    ROM[15] <= {8'h00, 10'h000}; // Unused
    // Set the rest to all zeros
    //ROM[16] <= 8'h00;
end

// write is unused
wire [ADDR_WIDTH-1:0] ADDRA = {ADDR_WIDTH{1'b0}}; 
wire                  WEA   = 1'b0; 
always @ (posedge clock)
begin
    if (WEA) begin
        ROM[ADDRA] <= DATAA;
    end
end
   
assign RADDR = addr_ptr;
   
reg [ROM_SIZE-1:0] RDATA;
always @ (posedge clock)
begin
     RDATA <= ROM[RADDR];
end

assign rdata_out = RDATA;
   
endmodule			      