#define DEBUG_MODE 0

#include <stdio.h>
#include <io.h>
#include <system.h>
#include <unistd.h>
#include "alt_types.h"
//#include "vip_func.h"
#include "xcvr_gpll_rcfg.h"
//#include "i2c.h"
//#include "ti_i2c.h"
#include "intel_fpga_i2c.h"


int main()
{
	int pix_rate;
	int tmds_clk_freq;
	int gpll_range = 20;
	int prev_gpll_range;
	int txpll_range = 40;
	int prev_txpll_range;
	int tmds_bit_clock_ratio = 0;
	int prev_tmds_bit_clock_ratio;
	int color_depth = 0;
	int prev_color_depth;
	int tx_hpd_req;
	int gpll_xcvr_rcfg = 1;
	int tx_os=0;
  
	IOWR(WD_TIMER_BASE, 0x1, 0x8); // stop the timer
	IOWR(TX_OS_PIO_BASE, 0, 0); // set to non-oversampling mode
    IOWR(TX_RST_PLL_PIO_BASE, 0, 1); // assert reset for TX PLL
	IOWR(TX_RST_PLL_PIO_BASE, 0, 0); // deassert reset for TX PLL
	IOWR(TX_HPD_ACK_PIO_BASE, 0, 0); // set TX HPD acknowledge bit to 0
	IOWR(EDID_RAM_ACCESS_PIO_BASE, 0, 0); //release access to edid RAM
	// Send acknowledge bit to pull down measure_valid_req
	IOWR(MEASURE_VALID_ACK_BASE, 0, 1);
	IOWR(MEASURE_VALID_ACK_BASE, 0, 0);

	intel_fpga_i2c_init(I2C_MASTER_BASE, 100000000);

	//printf("TPD158\n");
    intel_fpga_i2c_init(I2C_MASTER_TI_BASE, 100000000);

	while(1)
	{
		// Check if TX hot plug event has happened
		tx_hpd_req = IORD(TX_HPD_REQ_PIO_BASE,0);

		// Wait for measure_valid assertion
		while ((IORD(MEASURE_VALID_REQ_BASE,0) == 0) && (tx_hpd_req == 0))
		//while (IORD(MEASURE_VALID_PIO_BASE,0) == 0)
		{
			// Poll TX hpd signal
			tx_hpd_req = IORD(TX_HPD_REQ_PIO_BASE,0);
		}
		//tx_hpd_req = 1;
		// Placeholder for next revision of Bitec HDMI 2.0 FMC daughter card
		// The current revision has level shifter issue 
		// Check if TX hot plug event has happened
		//tx_hpd_req = IORD(TX_HPD_REQ_PIO_BASE,0);
		if (tx_hpd_req)
		{
			printf("TX hot plug event is detected...\n");

			// If it is 2.0 rate, resend SCDC bit as it is required by the 2.0 sink
			if(IORD(TMDS_BIT_CLOCK_RATIO_PIO_BASE, 0)==1){
				intel_fpga_i2c_write_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20, 0x3);
				printf("TMDS_STATUS  : %X\n", intel_fpga_i2c_read_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20));
			}

			IOWR(EDID_RAM_ACCESS_PIO_BASE, 0, 1); //gain access to edid RAM

			usleep(1000000); //wait for 1s to toggle RX HPD to initiate updated EDID information read
			IOWR(EDID_RAM_ACCESS_PIO_BASE, 0, 0); //release access to edid RAM

			// Send acknowledge bit to pull down tx_hpd_req
			IOWR(TX_HPD_ACK_PIO_BASE, 0, 1);
			IOWR(TX_HPD_ACK_PIO_BASE, 0, 0);                  
		}
		
		prev_tmds_bit_clock_ratio = tmds_bit_clock_ratio;
		tmds_bit_clock_ratio = IORD(TMDS_BIT_CLOCK_RATIO_PIO_BASE, 0);

		tmds_clk_freq = IORD(MEASURE_PIO_BASE,0);
		
		// Get pixel clock rate
		if (IORD(TMDS_BIT_CLOCK_RATIO_PIO_BASE, 0)==1){
			pix_rate = tmds_clk_freq << 2;
		} else {
			pix_rate = tmds_clk_freq;
		}
		// Do nothing if rate detect returns all 1's (invalid rate)
		if (pix_rate < 0xFFFFFF) {

			tx_os = TX_OS_ENC(pix_rate);
			IOWR(TX_OS_PIO_BASE, 0, tx_os);
			printf("tx_os:%d\n",tx_os);

			prev_color_depth = color_depth;
			color_depth = IORD(COLOR_DEPTH_PIO_BASE, 0);
			printf("color_depth:%d\n",color_depth);

			prev_gpll_range = gpll_range;
			gpll_range = GPLL_COMPARE(pix_rate);
			printf("gpll_range:%d\n",gpll_range);

			prev_txpll_range = txpll_range;
			txpll_range = TXPLL_COMPARE(pix_rate);
			printf("txpll_range:%d\n",txpll_range);

			//gpll_xcvr_rcfg = 1;
			gpll_xcvr_rcfg = prev_gpll_range != gpll_range || prev_txpll_range != txpll_range || prev_color_depth != color_depth || prev_tmds_bit_clock_ratio != tmds_bit_clock_ratio;
		} else {
			gpll_xcvr_rcfg = 0;
	 	}

		// Reconfig TX IOPLL and TX XCVR PLL
		if (gpll_xcvr_rcfg)
		{	
			if (tmds_bit_clock_ratio == 1){
				intel_fpga_i2c_write_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20, 0x3);
				 printf("TMDS_STATUS  : %X\n", intel_fpga_i2c_read_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20));
			} else {
				intel_fpga_i2c_write_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20, 0x0);
				 printf("TMDS_STATUS  : %X\n", intel_fpga_i2c_read_extended(I2C_MASTER_BASE, 0xA8 >> 1, 0x20));
			}

			// Reconfig TX IOPLL
			if ((prev_gpll_range != gpll_range) || prev_color_depth != color_depth || (prev_tmds_bit_clock_ratio != tmds_bit_clock_ratio))
			{
				if (DEBUG_MODE)
				{
					printf("GPLL current range = %d\n", gpll_range);
					printf("Color depth = %d\n", color_depth);
					printf("Pixel rate = %d\n", pix_rate);
					printf("Reconfiguring GPLL...\n");
				}

				GPLL_RECONFIG(gpll_range, color_depth);
				printf("Resetting GPLL...\n");
				IOWR(TX_RST_PLL_PIO_BASE, 0, 1);
				IOWR(TX_RST_PLL_PIO_BASE, 0, 0);
			}

			// Reconfig TX XCVR PLL
			//if (DEBUG_MODE)
			//{
				printf("TX PLL current range = %d\n", txpll_range);
				printf("Pixel rate = %d\n", pix_rate);                       
				printf("Reconfiguring TX PLL (FPLL) ...\n");
			//}
			printf("Resetting XCVR...\n");
			IOWR(TX_RST_XCVR_PIO_BASE, 0, 1);

			TX_PLL_RECONFIG(txpll_range);

			// Reset XCVR
			printf("Releasing XCVR reset...\n");
			IOWR(TX_RST_XCVR_PIO_BASE, 0, 0);
			printf("-----------------------------------\n");
        }
        
      // Send acknowledge bit to pull down measure_valid_req
      IOWR(MEASURE_VALID_ACK_BASE, 0, 1);
      IOWR(MEASURE_VALID_ACK_BASE, 0, 0);

	}

	return 0;
}



