# (C) 2001-2020 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


## *********************************************************************
##
##
## Bitec HDMI IP Core
## 
##
## All rights reserved. Property of Bitec.
## Restricted rights to use, duplicate or disclose this code are
## granted through contract.
## 
## (C) Copyright Bitec 2010,2011,2012,2013,2014
##     All rights reserved
##
## *********************************************************************
## Author         : $Author: Andy $ @ bitec-dsp.com
## Department     : 
## Date           : $Date: 2016-06-21 10:01:07 +0200 (Tue, 21 Jun 2016) $
## Revision       : $Revision: 981 $
## URL            : $URL: svn://10.9.0.1/hdmi/trunk/bitec_hdmi.sdc $
## *********************************************************************
## Description
##
## Timing constraints
## 
## *********************************************************************

## *********************************************************************
## Constrain all used DCFIFO at top level, instance by instance
## *********************************************************************
# constraints for DCFIFO sdc
#
# top-level sdc
# convention for module sdc apply_sdc_<module_name>
#
#
# common constraint setting proc
#
proc apply_sdc_dcfifo {hier_path} {
# gray_rdptr
apply_sdc_dcfifo_rdptr $hier_path
# gray_wrptr
apply_sdc_dcfifo_wrptr $hier_path
}
#
#
proc apply_sdc_dcfifo_for_ptrs {from_node_list to_node_list} {
# control skew for bits
set_max_skew -from $from_node_list -to $to_node_list -get_skew_value_from_clock_period src_clock_period -skew_value_multiplier 0.8
# path delay (exception for net delay)
if { ![string equal "quartus_syn" $::TimeQuestInfo(nameofexecutable)] } {
set_net_delay -from $from_node_list -to $to_node_list -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
}
#relax setup and hold calculation
set_max_delay -from $from_node_list -to $to_node_list 100
set_min_delay -from $from_node_list -to $to_node_list -100
}
#============================
# mstable propgation delay
#============================
proc apply_sdc_dcfifo_mstable_delay {from_node_list to_node_list} {
# mstable delay
if { ![string equal "quartus_syn" $::TimeQuestInfo(nameofexecutable)] } {
set_net_delay -from $from_node_list -to $to_node_list -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
}
}
#============================
# rdptr constraints
#============================
proc apply_sdc_dcfifo_rdptr {hier_path} {
# get from and to list
set from_node_list [get_keepers $hier_path|auto_generated|*rdptr_g*]
set to_node_list [get_keepers $hier_path|auto_generated|ws_dgrp|dffpipe*|dffe*]
apply_sdc_dcfifo_for_ptrs $from_node_list $to_node_list
# mstable
set from_node_mstable_list [get_keepers $hier_path|auto_generated|ws_dgrp|dffpipe*|dffe*]
set to_node_mstable_list [get_keepers $hier_path|auto_generated|ws_dgrp|dffpipe*|dffe*]
apply_sdc_dcfifo_mstable_delay $from_node_mstable_list $to_node_mstable_list
}
#============================
# wrptr constraints
#============================
proc apply_sdc_dcfifo_wrptr {hier_path} {
# control skew for bits
set from_node_list [get_keepers $hier_path|auto_generated|delayed_wrptr_g*]
set to_node_list [get_keepers $hier_path|auto_generated|rs_dgwp|dffpipe*|dffe*]
apply_sdc_dcfifo_for_ptrs $from_node_list $to_node_list
# mstable
set from_node_mstable_list [get_keepers $hier_path|auto_generated|rs_dgwp|dffpipe*|dffe*]
set to_node_mstable_list [get_keepers $hier_path|auto_generated|rs_dgwp|dffpipe*|dffe*]
apply_sdc_dcfifo_mstable_delay $from_node_mstable_list $to_node_mstable_list
}

# ============================
# Filter entity path
# ============================
proc apply_sdc_pre_dcfifo {entity_name} {
set inst_list [get_entity_instances $entity_name]
    foreach each_inst $inst_list {
        set tx_entity [get_entity_instances -nowarn bitec_hdmi_tx]

        if {[string equal -length [string length $tx_entity] $tx_entity $each_inst]} {
            regsub ***=${tx_entity}| $each_inst "" each_inst
            apply_sdc_dcfifo "u_bitec_hdmi_tx|${each_inst}"
        }
    }
}

apply_sdc_pre_dcfifo dcfifo

proc apply_set_mcp {path_name} {
  set path_list [get_keepers -nowarn  $path_name]
  if {[get_collection_size $path_list] > 0} {
     foreach_in_collection path  $path_list {
        set_multicycle_path -end -setup -to $path 10
        set_multicycle_path -end -hold -to $path 9
     }
   }
}

apply_set_mcp *auxiliary_encoder.audio.audio|phase_wrap
apply_set_mcp *auxiliary_encoder.audio.audio|cntr_1m[*]
apply_set_mcp *auxiliary_encoder.audio.audio|CTS_valid
