// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// 
// This ROM stores the address offset and bitmask for GXB RX that needs reconfig
//
// addr_ofst - the address offset that needs reconfig
// field_bitmask - the bit that needs reconfig (eg. 8'h40 means only bit 6 needs reconfig)
//
module mr_rom_rx_dprioaddr_bitmask #(
    parameter ROM_SIZE        = 18, // number of bits per rom address - [17:8] address offset, [7:0] field bitmask   
    parameter TOTAL_ROM_DEPTH = 16, // ROM_DEPTH_FOR_EACH_MIF_RANGE - total attributes that needs reconfig
    parameter ADDR_WIDTH      = 3   // alt_clogb2(7)
) (
    input  wire                  clock,
    input  wire [ADDR_WIDTH-1:0] addr_ptr,
    output wire [ROM_SIZE-1:0]   rdata_out
);

reg  [ROM_SIZE-1:0]   ROM [0:TOTAL_ROM_DEPTH-1];
wire [ROM_SIZE-1:0]   DATAA = {ROM_SIZE{1'b0}};
wire [ADDR_WIDTH-1:0] RADDR;

// **** PERL SCIPT TO GENERATE BEGIN **** //   
initial begin
            // ADDR_OFST, FIELD_BITMASK
    ROM[ 0] <= {11'h13A, 8'hFF}; // [7:6]-cdr_pll_fref_clklow_div=1(2'h0); [5:3]-cdr_pll_pd_l_counter=2(3'h3); [2:0]-cdr_pll_pfd_l_counter=2(3'h3);
    ROM[ 1] <= {11'h13B, 8'hFF}; // [7:0]-cdr_pll_mcnt_div=20(8'h14);
    ROM[ 2] <= {11'h13C, 8'hFD}; // [7:4]-cdr_pll_set_cdr_vco_speed_pciegen3=cdr_vco_max_speedbin_pciegen3(4'h2); [3:2]-cdr_pll_ncnt_div=1(2'h0); [0:0]-pma_rx_deser_rst_n_adapt_odi=no_rst_adapt_odi(1'h1);
    ROM[ 3] <= {11'h159, 8'hFF}; // [7:0]-cdr_pll_set_cdr_input_freq_range=39(8'h27);
    /*ROM[ 4] <= {11'h000, 8'h00};
    ROM[ 5] <= {11'h000, 8'h00};
    ROM[ 6] <= {11'h000, 8'h00};
    ROM[ 7] <= {11'h000, 8'h00};
    ROM[ 8] <= {11'h000, 8'h00};
    ROM[ 9] <= {11'h000, 8'h00};
    ROM[10] <= {11'h000, 8'h00};
    ROM[11] <= {11'h000, 8'h00};
    ROM[12] <= {11'h000, 8'h00};
    ROM[13] <= {11'h000, 8'h00};
    ROM[14] <= {11'h000, 8'h00};
    ROM[15] <= {11'h000, 8'h00};   */
end
// **** PERL SCIPT TO GENERATE END ****** //

// write is unused
wire [ADDR_WIDTH-1:0] ADDRA = {ADDR_WIDTH{1'b0}}; 
wire                  WEA   = 1'b0; 
always @ (posedge clock)
begin
    if (WEA) begin
        ROM[ADDRA] <= DATAA;
    end
end
   
assign RADDR = addr_ptr;
   
reg [ROM_SIZE-1:0] RDATA;
always @ (posedge clock)
begin
     RDATA <= ROM[RADDR];
end

assign rdata_out = RDATA;

endmodule
