// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// 40G ethernet top level file
// This module contains the MAC, PCS, PMA, and an optional avalon streaming adapter

/*
               |---------|    128b    |-----|    132b    |-----|   264b    |-----|
256b data <--->| Adapter |<---------->| MAC |<---------->| PCS |<--------->| PMA |<---> 4x serial lanes
               |---------|            |-----|            |-----|           |-----|
*/

`timescale 1 ps / 1 ps
module ex_40g_s10_alt_e40_181_jke64uy #(
        parameter TARGET_CHIP           = 6,
        parameter SIM_EMULATE           = 1'b0,
        parameter SIM_SHORT_AM          = 1'b0,
        parameter SYNOPT_LINK_FAULT     = 1'b0,
        parameter DYNAMIC_SKEW          = 1'b0,
        parameter SPEED_CONFIG          = 40,
        parameter EXT_TX_PLL            = 1,
        parameter PHY_REFCLK            = 1,
        parameter SYNOPT_TXCRC_INS      = 1,
        parameter SYNOPT_PREAMBLE_PASS  = 0,
        parameter SYNOPT_MAC_STATS      = 1,
        parameter SYNOPT_READY_LATENCY  = 0,
        parameter SYNOPT_FLOW_CONTROL   = 0,
        parameter SYNOPT_NUMPRIORITY    = 8,
        parameter SYNOPT_STRICT_SOP     = 0,    // UNH SFD compliance feature
        parameter SYNOPT_SYNC_E         = 0,
        parameter ENABLE_ADME           = 0,
        parameter WORDS                 = 2,
        parameter STATUS_CLK_KHZ        = 100000,     // clk_status rate in Mhz
        parameter ENABLE_KR4            = 0, 
        parameter SYNTH_AN              = 0,
        parameter SYNTH_SEQ             = 0,
        parameter LINK_TIMER_KR         = 513,
        parameter AN_TECH               = 8,
        parameter AN_CHAN               = 2,
        parameter AN_PAUSE              = 2,
        parameter SYNTH_LT              = 0,
        parameter BERWIDTH              = 4,
        parameter VMAXRULE              = 30,
        parameter VMINRULE              = 6,
        parameter VODMINRULE            = 14,
        parameter VPOSTRULE             = 25,
        parameter VPRERULE              = 16,
        parameter PREMAINVAL            = 30,
        parameter PREPOSTVAL            = 0,
        parameter PREPREVAL             = 0,
        parameter INITMAINVAL           = 25,
        parameter INITPOSTVAL           = 13,
        parameter INITPREVAL            = 3,
        parameter TRNWTWIDTH            = 8,
        parameter USE_DEBUG_CPU         = 0,
        parameter SYNTH_FEC             = 0,
        parameter CAPABLE_FEC           = 0,
        parameter ENABLE_FEC            = 0,
	parameter RX_PLL_TYPE           = "IOPLL"			     
) (
        output                      clk_rxmac,
        output      [5:0]           l2_rx_error,
        output                      l2_rx_valid,
        output                      l2_rx_startofpacket,
        output                      l2_rx_endofpacket,
        output      [3:0]           l2_rx_empty,
        output      [64*WORDS-1:0]  l2_rx_data,
        output                      clk_txmac,
        input                       clk_txmac_in,
        input                       l2_tx_error,
        input                       l2_tx_startofpacket,
        input                       l2_tx_endofpacket,
        input                       l2_tx_valid,
        output                      l2_tx_ready,
        input       [3:0]           l2_tx_empty,
        input       [64*WORDS-1:0]  l2_tx_data,
        output                      tx_lanes_stable,
        output                      rx_pcs_ready,
        input                       clk_ref,
        input                       csr_rst_n,
        input                       tx_rst_n,
        input                       rx_rst_n,
        output                      rx_block_lock,
        output                      rx_am_lock,
        input                       tx_serial_clk,
        input                       tx_pll_locked,
        input                       reconfig_clk,
        input                       reconfig_reset,
        input                       reconfig_write,
        input                       reconfig_read,
        input       [12:0]          reconfig_address,
        input       [31:0]          reconfig_writedata,
        output      [31:0]          reconfig_readdata,
        output                      reconfig_waitrequest,
        output      [3:0]           tx_serial,
        input       [3:0]           rx_serial,
        output                      l2_txstatus_valid,
        output      [39:0]          l2_txstatus_data,
        output      [6:0]           l2_txstatus_error,
        output                      l2_rxstatus_valid,
        output      [39:0]          l2_rxstatus_data,
        input       [63:0]          l2_tx_preamble,
        output      [63:0]          l2_rx_preamble,
        input                       clk_status,
        input                       status_write,
        input                       status_read,
        input       [15:0]          status_addr,
        input       [31:0]          status_writedata,
        output      [31:0]          status_readdata,
        output                      status_readdata_valid,
        output                      status_waitrequest,
        output                      clk_rx_recover,
        output                      remote_fault_status,
        output                      local_fault_status,
        output                      unidirectional_en,
        output                      link_fault_gen_en,

        // Flow control interface
        input  [SYNOPT_NUMPRIORITY-1:0] pause_insert_tx0,
        input  [SYNOPT_NUMPRIORITY-1:0] pause_insert_tx1,
        output [SYNOPT_NUMPRIORITY-1:0] pause_receive_rx
    );

    assign rx_am_lock = rx_pcs_ready;


    //------------------------------------------
    //------------- TX MAC PLL  ----------------
    //------------------------------------------
    wire tx_fabric_pll_locked;
    generate
        if (EXT_TX_PLL) begin : tx_pll_gen
            assign clk_txmac = clk_txmac_in;
            assign tx_fabric_pll_locked = 1'b1;
        end else begin        : tx_pll_gen
            s10_fpll_tx tx_pll (
                .pll_refclk0    (clk_ref),
                .outclk_div1    (clk_txmac),
                .pll_locked     (tx_fabric_pll_locked),
                .pll_cal_busy   (/* Unconnected */)
            );
        end
    endgenerate

    //------------------------------------------
    //------------- Core Module ----------------
    //------------------------------------------

    wire [2*66*WORDS-1:0]   tx_pcs_data;
    wire [3:0]              tx_pcs_valid;
    wire [3:0]              tx_err_ins;
    
    wire [2*66*WORDS-1:0]   rxpcs_din;
    wire                    rxpcs_frm_err_sclr;
    wire                    rx_fifo_soft_purge;
    wire [3:0]              rxpcs_dout_sticky_frm_err;
    wire [3:0]              rxpcs_dout_word_locked;
    wire                    rxpcs_dout_deskew_locked;
    wire                    rxpcs_dout_align_locked;
    wire [3:0]              rx_bitslip;
wire                inhibit_slip;
wire                inhibit_rx_reset;
wire [3:0]          rx_bitslip_xcvr = inhibit_slip? 4'b0000 : rx_bitslip;
    wire                    hi_ber_raw;
    wire                    blk_err;
    wire [3:0]              rfifo_rd_en;
    wire                    xcvr_rx_dig_rst_req;
    wire                    pma_reset;
    wire [3:0]              rx_ready;
    wire [3:0]              tx_ready;
    wire [3:0]              rx_data_valid;
    wire                    rx_clk;
    wire [3:0]              rx_seriallpbken;
    wire                    tx_clk  = clk_txmac;
    wire [3:0]              tx_digitalreset;
    wire [3:0]              rx_digitalreset;
    wire                    rx_pcs_pll_locked;

    wire [3:0]              rx_is_lockedtodata;
    wire [3:0]              rx_is_lockedtoref;
    wire [3:0]              rx_analogreset_stat;
    wire [3:0]              tx_analogreset_stat;
    wire [3:0]              rx_digitalreset_stat;
    wire [3:0]              rx_digitalreset_stat_pma,rx_digitalreset_stat_trs_kr;
    wire [3:0]              tx_digitalreset_stat;
    wire [3:0]              tx_fifo_full;
    wire [3:0]              tx_fifo_empty;
    wire [3:0]              tx_fifo_pfull;
    wire [3:0]              tx_fifo_pempty;
    wire [3:0]              rx_fifo_full;
    wire [3:0]              rx_fifo_empty;
    wire [3:0]              rx_fifo_pfull;
    wire [3:0]              rx_fifo_pempty;

    wire  [3:0]             rx_clkout;
    wire  [3:0]             rx_pma_iqtxrx_clkout;   
   
    wire  [3:0]             tx_clkout;
    //--Some connections are different for KR mode
   wire [3:0]  tx_coreclk_to_pma,rx_coreclk_to_pma;
   wire [3:0]   tx_coreclkin, rx_coreclkin;
   wire        reconfig_write_pma, reconfig_write_arb;
   wire        reconfig_read_pma, reconfig_read_arb;
   wire [12:0] reconfig_address_pma,reconfig_address_arb;
   wire [31:0] reconfig_writedata_pma, reconfig_writedata_arb;
   wire        reconfig_waitrequest_pma, reconfig_waitrequest_arb;
   wire        rx_set_locktoref; 
   wire rx_set_locktodata, rx_set_locktoref_core;
   wire [3:0]  rx_enh_clr_errblk_count;
   wire [3:0]  rx_enh_blk_lock;
   wire         tx_kr4_ready,rx_kr4_ready;
   wire [3:0]   tx_pma_ready,rx_pma_ready;
   wire [3:0]   rx_cdr_locked_kr4;
   wire         start_reset_ctrl;
   wire         start_reset_ctrl_rx, start_reset_ctrl_tx_d ;
   wire         start_reset_ctrl_tx_aib,start_reset_ctrl_tx_pcs;
   wire  [3:0] rx_cal_busy;
   wire  [3:0] tx_cal_busy;
   wire        pll_recal_done;

    wire rx_pll_recal_done_sync;
    wire rx_pcs_pll_locked_sync;
    reg rx_pcs_pll_sync_status_clk;
   wire [3:0] rx_transfer_ready,rx_aib_reset, rx_pcs_reset; 
   wire [3:0] tx_transfer_ready,tx_aib_reset, tx_pcs_reset; 
   wire       reset_aib_ack,reset_pcs_ack;

    alt_e40_synchronizer  rx_pcs_pll_lk_sync (
        .clk    (clk_status),
        .din    (rx_pcs_pll_locked),
        .dout   (rx_pcs_pll_locked_sync)
    	);
   alt_e40_synchronizer  pll_recal_done_sync (
       .clk    (clk_status),
        .din   (pll_recal_done),
        .dout  (rx_pll_recal_done_sync)
	);

   always @(posedge clk_status)
       rx_pcs_pll_sync_status_clk <= rx_pcs_pll_locked_sync & rx_pll_recal_done_sync;

   localparam  kr4_layer = SYNTH_SEQ || SYNTH_FEC ; // SEQ indicates AN/LT ON

    alt_e40_core_jke64uy  #(
        .SIM_EMULATE            (SIM_EMULATE),
        .SIM_SHORT_AM           (SIM_SHORT_AM),
        .SYNOPT_LINK_FAULT      (SYNOPT_LINK_FAULT),
        .DYNAMIC_SKEW           (DYNAMIC_SKEW),
        .TARGET_CHIP            (TARGET_CHIP),
        .EXT_TX_PLL             (EXT_TX_PLL),
        .SYNOPT_TXCRC_INS       (SYNOPT_TXCRC_INS),
        .SYNOPT_PREAMBLE_PASS   (SYNOPT_PREAMBLE_PASS),
        .SYNOPT_MAC_STATS       (SYNOPT_MAC_STATS),
        .SYNOPT_FLOW_CONTROL    (SYNOPT_FLOW_CONTROL),
        .SYNOPT_NUMPRIORITY     (SYNOPT_NUMPRIORITY),
        .SYNOPT_STRICT_SOP      (SYNOPT_STRICT_SOP),
        .SYNOPT_READY_LATENCY   (SYNOPT_READY_LATENCY),
        .ENABLE_KR4             (kr4_layer), 
        .SYNTH_AN               (SYNTH_AN),
        .SYNTH_LT               (SYNTH_LT),
        .SYNTH_SEQ              (SYNTH_SEQ),
        .SYNTH_FEC              (SYNTH_FEC),
        .LINK_TIMER_KR          (LINK_TIMER_KR),
        .BERWIDTH               (BERWIDTH),
        .TRNWTWIDTH             (TRNWTWIDTH),
        .MAINTAPWIDTH           (5),
        .POSTTAPWIDTH           (6),
        .PRETAPWIDTH            (5),
        .VMAXRULE               (VMAXRULE),
        .VMINRULE               (VMINRULE),
        .VODMINRULE             (VODMINRULE),
        .VPOSTRULE              (VPOSTRULE),
        .VPRERULE               (VPRERULE),
        .PREMAINVAL             (PREMAINVAL),
        .PREPOSTVAL             (PREPOSTVAL),
        .PREPREVAL              (PREPREVAL),
        .INITMAINVAL            (INITMAINVAL),
        .INITPOSTVAL            (INITPOSTVAL),
        .INITPREVAL             (INITPREVAL),
        .USE_DEBUG_CPU          (USE_DEBUG_CPU),
        .AN_CHAN                (AN_CHAN),
        .AN_PAUSE               (AN_PAUSE),
        .AN_TECH                (AN_TECH),
        .CAPABLE_FEC            (CAPABLE_FEC),
        .ENABLE_FEC             (ENABLE_FEC),
        .STATUS_CLK_KHZ         (STATUS_CLK_KHZ)
    ) eth_core (
        .tx_lanes_stable        (tx_lanes_stable),
        .avmm_write             (status_write),
        .avmm_read              (status_read),
        .avmm_address           (status_addr),
        .avmm_write_data        (status_writedata),
        .avmm_read_data         (status_readdata),
        .avmm_read_data_valid   (status_readdata_valid),
        .avmm_waitrequest       (status_waitrequest),
        // Incoming rcfg REQ from TOP
        .reconfig_write_top(reconfig_write),
        .reconfig_read_top(reconfig_read),
        .reconfig_address_top(reconfig_address),
        .reconfig_writedata_top(reconfig_writedata),
        // Incoming from HSSI 
        .reconfig_readdata_pma(reconfig_readdata),
        .reconfig_waitrequest_pma(reconfig_waitrequest_pma),
        // ARB output to HSSI/TOP
        .reconfig_write_arb(reconfig_write_arb),
        .reconfig_read_arb(reconfig_read_arb),
        .reconfig_address_arb(reconfig_address_arb),
        .reconfig_writedata_arb(reconfig_writedata_arb),
        .reconfig_waitrequest_arb(reconfig_waitrequest_arb),

        .rx_pempty              (rx_fifo_pempty),
        .rx_pfull               (rx_fifo_pfull),
        .rx_empty               (rx_fifo_pempty),
        .rx_full                (rx_fifo_full),
        .rx_set_locktoref       (rx_set_locktoref_core),
        .rx_set_locktodata      (rx_set_locktodata),
        .rx_enh_clr_errblk_count(rx_enh_clr_errblk_count),
        .rx_enh_blk_lock        (rx_enh_blk_lock),
        .tx_pempty              (tx_fifo_pempty),
        .tx_pfull               (tx_fifo_pfull),
        .tx_empty               (tx_fifo_empty),
        .tx_full                (tx_fifo_full),
        .tx_pll_locked          (tx_pll_locked),
        .rx_seriallpbken        (rx_seriallpbken),
        .tx_digitalreset        (!tx_pma_ready[1]),
        .tx_digitalreset_pma    (|tx_digitalreset),
        .rx_digitalreset_pma    (|rx_digitalreset),

        .pause_insert_tx0       (pause_insert_tx0),
        .pause_insert_tx1       (pause_insert_tx1),
        .pause_receive_rx       (pause_receive_rx),

        .l2_tx_error            (l2_tx_error),
        .l2_tx_startofpacket    (l2_tx_startofpacket),
        .l2_tx_endofpacket      (l2_tx_endofpacket),
        .l2_tx_valid            (l2_tx_valid),
        .l2_tx_ready            (l2_tx_ready),
        .l2_tx_empty            (l2_tx_empty),
        .l2_tx_data             (l2_tx_data),
        .tx_fabric_pll_locked   (tx_fabric_pll_locked),

        .l2_txstatus_data       (l2_txstatus_data),
        .l2_txstatus_valid      (l2_txstatus_valid),
        .l2_txstatus_error      (l2_txstatus_error),

        .l2_tx_preamble         (l2_tx_preamble),
        .l2_rx_preamble         (l2_rx_preamble),

        .remote_fault_status    (remote_fault_status),
        .local_fault_status     (local_fault_status),
        .unidirectional_en      (unidirectional_en),
        .link_fault_gen_en      (link_fault_gen_en),

        .l2_rxstatus_data       (l2_rxstatus_data),
        .l2_rxstatus_valid      (l2_rxstatus_valid),

        .l2_rx_error            (l2_rx_error),
        .l2_rx_valid            (l2_rx_valid),
        .l2_rx_startofpacket    (l2_rx_startofpacket),
        .l2_rx_endofpacket      (l2_rx_endofpacket),
        .l2_rx_data             (l2_rx_data),
        .l2_rx_empty            (l2_rx_empty),

	.soft_tx_rst (soft_tx_rst),
        .inhibit_rx_reset       (inhibit_rx_reset), 
        .inhibit_slip           (inhibit_slip), 
        .tx_rst                 (~tx_rst_n),
        .tx_clk                 (tx_clk),
        .clk_avmm               (clk_status),
        .tx_pcs_data            (tx_pcs_data),
        .tx_pcs_valid           (tx_pcs_valid),
        .tx_err_ins             (tx_err_ins), 
        .rx_pcs_pll_locked      (rx_pcs_pll_sync_status_clk),

        .rx_clk                 (rx_clk),
        .rx_cdr_refclk          (clk_ref),
        .rx_rst                 (~rx_rst_n),
        .rx_din                 (rxpcs_din),
        .csr_rst_n              (csr_rst_n),
        .rpcs_deskew_locked     (rxpcs_dout_deskew_locked),
        .rpcs_align_locked      (rxpcs_dout_align_locked),
        .rx_is_lockedtodata     (rx_is_lockedtodata),
        .rx_is_lockedtoref      (rx_is_lockedtoref),
        .rx_analogreset_stat    (rx_analogreset_stat)  ,
        .tx_analogreset_stat    (tx_analogreset_stat)  ,
        .rx_digitalreset_stat   (rx_digitalreset_stat)  ,
        .tx_digitalreset_stat   (tx_digitalreset_stat)  ,
        .rx_cal_busy            (rx_cal_busy),
        .tx_cal_busy            (tx_cal_busy),
        .rpcs_word_locked       (rxpcs_dout_word_locked),
        .rpcs_bit_slip          (rx_bitslip),
        .rpcs_hi_ber            (hi_ber_raw),
        .rpcs_blk_err           (blk_err),
        .rfifo_rd_en            (rfifo_rd_en),
        .rx_din_valid           (rx_data_valid),

        .tx_clkout              (tx_clkout),
        .rx_clkout              (rx_clkout),
        .tx_coreclkin           (tx_coreclkin),
        .rx_coreclkin           (rx_coreclkin),

        .tx_digital_reset_status(|tx_digitalreset_stat),
        .xcvr_rx_dig_rst_out    (xcvr_rx_dig_rst_req),
        .native_phy_reset       (pma_reset),
        .rx_pcs_ready           (rx_pcs_ready),
        .rx_ready               (rx_ready[0]),
        .tx_pma_ready           (tx_pma_ready),
        .tx_kr4_ready           (tx_kr4_ready),
        .rx_pma_ready           (rx_pma_ready),
        .rx_kr4_ready           (rx_kr4_ready),
        .rx_cdr_locked_kr4      (rx_cdr_locked_kr4),
        .start_reset_ctrl       (start_reset_ctrl),
        .start_reset_ctrl_rx    (start_reset_ctrl_rx),
        .start_reset_ctrl_tx_d  (start_reset_ctrl_tx_d),
        .start_reset_ctrl_tx_aib(start_reset_ctrl_tx_aib),
        .start_reset_ctrl_tx_pcs(start_reset_ctrl_tx_pcs),
        .reset_aib_ack          (reset_aib_ack),
        .reset_pcs_ack          (reset_pcs_ack),
//PLL calibration done signal for KR
	.rxpll_recal_done         (pll_recal_done)
    );
    assign rx_block_lock = &rxpcs_dout_word_locked;

    //------------------------------------------
    //------------- RX PCS Clock ---------------
    //------------------------------------------
//-------------------------------------------------------------------------------------
// ----------------- Cascding and Calibration details----------------------------------
// on RX side CDR PLL is cascaded to either IOPLL or FPLL
// Here are the details of cascading and the solution we are using for cascading
// In S10 40G Ethernet on RX side we have CDR->PLL cascading, PLL can be IOPLL or FPLL.
// The complication is cascading where refclk of downgrade PLL is coming from CDR, 
// which is not stable after powerup so we need to recalibrate the PLL after powerup. 
// Remember in S10 we do need recalibration of IOPLL which was not the case for A10 IOPLL
// | --------|   |------------------| 
// |  CDR PLL|==>|refclk  IOPLL/FPLL|==> 312.5 Mhz clock (rx_coreclkin and core logic)
// |---------| 	 |------------------|
//   
// refclk in case of CDR->IOPLL cascading is rx_clkout (div33 clockout) = 156.25 Mhz
// refclk in case of CDR->FPLL  cascading is rx_pma_iqtxrxclkout (div40 clk) = 257.8125 Mhz    
// alt_e40_pll_recal.sv implements the recalibration logic
// Recalibration will kicked off at the assertion of either csr_rst_n or 
// soft_csr_rst (aka sys_rst/eio_sys_rst)
// In the logic below pma_reset is used pma_reset is native_phy_reset coming from the reset controller 
// and is asserted when csr_rst_n or sys_rst is asserted.  
// Recalibration sequence will be 
// Initially put CDR in locktoref mode by setting rx_set_locktoref to "1"
// wait CDR to lock to locktoref
// for stable locktoref wait for 20us
// Then start IOPLL/FPLL recalibration
// once recal is done switch CDR to auto mode
// Recalibration done is indicated by pll_recal_done
// Recal happens only once 
//-------------------------------------------------------------------------------------
   wire       int_rx_is_lockedtoref;
   wire       sync_rx_is_lockedtoref;   
   wire       rcfg_write; // AVMM write
   wire       rcfg_read; // AVMM read
   wire [10:0] rcfg_address; // AVMM address
   wire [31:0] rcfg_wrdata; // AVMM write data
   wire [31:0] rcfg_rddata; // AVMM read data
   wire        rcfg_wtrqst;    // AVMM wait request
   
   
   assign int_rx_is_lockedtoref = &rx_is_lockedtoref;

      // synchronize rx_is_lockedtoref wrt clk_status
   alt_e40_status_sync 
     #(.WIDTH(1))
   locktoref_sync (
    .clk  (clk_status),
    .din  (int_rx_is_lockedtoref),
    .dout (sync_rx_is_lockedtoref)
);

 // Generate rx_is_lockedtodata_stat signal
 // if rx_is_lockedtodata is "1" for 4 us for L-tile and 5us for H-tile 
 // that indicates stable rx_lockedtodata
 // generate rx_is_lockedtodata_stat signal
 // with cnt_lock = 9 and mgmt_clk = 100 mhz (10ns) = 10 *512 = 5.12us
   
   // synchronize rx_is_lockedtodata wrt clk_status
   alt_e40_status_sync 
     #(.WIDTH(1))
   locktodata_sync (
		   .clk  (clk_status),
		   .din  (&rx_is_lockedtodata),
		   .dout (sync_rx_is_lockedtodata)
		   );
	
	  
  localparam CNTWIDTH = SIM_EMULATE ? 3: 9;  
  reg [CNTWIDTH-1:0] cnt_lock;
  reg       cnt_max;

   // Counter for counting 5us for rx_lockedtodata
   always @(posedge clk_status or posedge pma_reset)
     begin
	if (pma_reset) 
	  begin
	     cnt_max  <= 1'b0;
	     cnt_lock <= {CNTWIDTH{1'b0}};
	  end
	else
	  begin
	     cnt_max <= (&cnt_lock);
	     if (~sync_rx_is_lockedtodata)      
	       cnt_lock <= {CNTWIDTH{1'b0}};
	     else if (&cnt_lock)  
	       cnt_lock <= cnt_lock;
	       else
	       cnt_lock <= cnt_lock + 1'b1;
	  end // else: !if(mgmt_reset)
     end // always @ (posedge mgmt_clk or posedge mgmt_reset)
   
   assign rx_is_lockedtodata_stat = (cnt_max) ? 1'b1 : 1'b0;
   wire recal_busy;
   wire fpll_cal_busy;
   
   assign recal_busy = (RX_PLL_TYPE == "IOPLL") ? 1'b0 : fpll_cal_busy;

	     
// PLL recalibration module to recalibrate IOPLL/FPLL
// 

   alt_e40_pll_recal 
     #(
       .RX_PLL_TYPE(RX_PLL_TYPE)
       ) recal_rxpll 
       (
	.mgmt_clk  (clk_status),
        .mgmt_reset(pma_reset),
        .cal_busy    (recal_busy),
        .rx_is_lockedtoref (sync_rx_is_lockedtoref),
	.rx_set_locktoref_core (rx_set_locktoref_core),				  
	.rx_set_locktoref (rx_set_locktoref),
        .recal_is_done      (pll_recal_done),

        .rcfg_write(rcfg_write),
        .rcfg_read (rcfg_read),
        .rcfg_address(rcfg_address),
        .rcfg_wrdata (rcfg_wrdata),
        .rcfg_rddata (rcfg_rddata),
        .rcfg_wtrqst (rcfg_wtrqst)
    );
   
// IOPLL or FPLL as RX PLL
   generate
      if (RX_PLL_TYPE == "FPLL") begin : FPLL_INST
	 // Instantiate FPLL instance, for CDR->FPLL cascade need to use rx_pma_iqtsrx_clkout port
	 s10_fpll_rx rx_pll 
	   (           
		       .outclk_div1 (rx_clk), // 312.5  MHz
		       .pll_cal_busy (fpll_cal_busy),
		       .pll_locked (rx_pcs_pll_locked),
		       .pll_refclk0(rx_pma_iqtxrx_clkout[1]), // 257.8125 MHz
                       .reconfig_write0(rcfg_write),
                       .reconfig_read0 (rcfg_read),
                       .reconfig_address0    (rcfg_address),
                       .reconfig_writedata0  (rcfg_wrdata),
                       .reconfig_readdata0   (rcfg_rddata),
                       .reconfig_waitrequest0(rcfg_wtrqst),
		       .reconfig_clk0  (clk_status),
                       .reconfig_reset0(pma_reset)				      

		       );
      end // block: FPLL_INST
      else begin : IOPLL_inst
	 //wire [10:0]  reconfig_from_iopll; // reconfig_from_pll fro IOPLL
	 wire [29:0] reconfig_to_iopll;//   reconfig_to_pll to IOPLL   
	 wire 	     rx_pll_reset = (pll_recal_done) ? ~rx_is_lockedtodata_stat : ~&rx_is_lockedtoref;
//wire 	     rx_pll_reset = |rx_digitalreset;
     assign rcfg_rddata = 32'd0;

    s10_iopll_rx rx_pll 
      (
       .rst        (rx_pll_reset),
       .refclk     (rx_clkout[0]),         // 156.25 MHz
       .locked     (rx_pcs_pll_locked),
       .outclk_0   (rx_clk),                // 312.5  MHz
	.permit_cal (~rx_pll_reset)
       //.reconfig_from_pll (reconfig_from_iopll),
       //.reconfig_to_pll (reconfig_to_iopll)			 
       );
	 
	 s10_iopll_reconfig  iopll_reconfig
	   (
	    .mgmt_waitrequest(rcfg_wtrqst),
	    .mgmt_write(rcfg_write),
	    .mgmt_read(rcfg_read),
	    .mgmt_writedata(rcfg_wrdata[7:0]),
	    .mgmt_readdata(),
	    .mgmt_address(rcfg_address[9:0]),
	    .mgmt_clk(clk_status),
	    .mgmt_reset(pma_reset),
	    .reconfig_from_pll(11'b0),
	    .reconfig_to_pll(reconfig_to_iopll)	 
	    );
	 
      end // block: IOPLL_inst
   endgenerate
   

    assign clk_rxmac = rx_clk;

    //------------------------------------------
    //-------- PMA and reset controller --------
    //------------------------------------------
    // .reset --> will initiate complete reset sequence 
    // .pll_locked --> if down, reset controller will assert TX D reset
    // .rx_cal_busy --> if up, reset controller will assert Rx A/D reset

    wire  [3:0] tx_analogreset;
    wire  [3:0] rx_analogreset;
    wire  [3:0] digital_rx_ready;
    s10_xcvr_reset_controller pma_reset_controller (
        .clock                  (reconfig_clk),         // 100 MHz
        .reset                  (pma_reset | start_reset_ctrl),            // Sync to clock
        .tx_analogreset         (tx_analogreset),
        .tx_digitalreset        (tx_digitalreset),
        .tx_ready               (tx_pma_ready),
        .pll_locked             (tx_pll_locked & start_reset_ctrl_tx_d & ~soft_tx_rst & tx_rst_n & tx_fabric_pll_locked ),
        .pll_select             (1'b0),
        .tx_cal_busy            (tx_cal_busy),
        .tx_analogreset_stat    (tx_analogreset_stat),
        .tx_digitalreset_stat   (tx_digitalreset_stat),
        .rx_analogreset         (rx_analogreset),
        .rx_digitalreset        (rx_digitalreset),
        .rx_ready               (rx_pma_ready),
        .rx_is_lockedtodata     (digital_rx_ready),
        .rx_cal_busy            (rx_cal_busy | {4{start_reset_ctrl_rx}}),
        .rx_analogreset_stat    (rx_analogreset_stat),
        .rx_digitalreset_stat   (rx_digitalreset_stat)
    );

   generate 
    if (kr4_layer) begin : KR_CONNECTIONS
       assign tx_coreclk_to_pma = tx_coreclkin;
       assign rx_coreclk_to_pma = rx_coreclkin;
       assign reconfig_write_pma = reconfig_write_arb;
       assign reconfig_read_pma  = reconfig_read_arb;
       assign reconfig_address_pma = reconfig_address_arb;
       assign reconfig_writedata_pma = reconfig_writedata_arb;
       assign reconfig_waitrequest = reconfig_waitrequest_arb;
       assign tx_ready             = {4{tx_kr4_ready}}; 
       assign rx_ready             = {4{rx_kr4_ready}}; 
       assign digital_rx_ready     = xcvr_rx_dig_rst_req & !inhibit_rx_reset? 4'b0000 : rx_cdr_locked_kr4;
    end else begin : NONKR_CONNECTIONS
       assign tx_coreclk_to_pma = {4{tx_clk}};
       assign rx_coreclk_to_pma = {4{rx_clk}};
       assign reconfig_write_pma = reconfig_write;
       assign reconfig_read_pma  = reconfig_read;
       assign reconfig_address_pma = reconfig_address;
       assign reconfig_writedata_pma = reconfig_writedata;
       assign reconfig_waitrequest = reconfig_waitrequest_pma;
       assign tx_ready             = tx_pma_ready; 
       assign rx_ready             = rx_pma_ready; 
       assign digital_rx_ready     = xcvr_rx_dig_rst_req & !inhibit_rx_reset? 4'b0000 : rx_is_lockedtodata;
    end 
   endgenerate
    wire [3:0]              tx_dll_lock;
    alt_e40_pma_jke64uy  #(
        .ENABLE_KR4             (kr4_layer)
     )    pma_40g (
        .tx_pcs_reset               (tx_pcs_reset),
        .rx_pcs_reset               (rx_pcs_reset),
        .tx_aib_reset               (tx_aib_reset),
        .rx_aib_reset               (rx_aib_reset),
        .tx_transfer_ready          (tx_transfer_ready),
        .rx_transfer_ready          (rx_transfer_ready),
        .rx_bitslip                 (rx_bitslip_xcvr),           // Async
        .rx_enh_clr_errblk_count    (rx_enh_clr_errblk_count),
        .rx_enh_blk_lock            (rx_enh_blk_lock),
        .rx_cdr_refclk0             (clk_ref),              // 644.53125 MHz
        .rx_clkout                  (rx_clkout),            // div 33 clk, 156.25 MHz
	.rx_pma_iqtxrx_clkout       (rx_pma_iqtxrx_clkout), // div 40 clk, 257.8125, required when RX PLL is used as FPLL
        .rx_coreclkin               (rx_coreclk_to_pma),    // 312.5 MHz for 40G/156 for AN/322 for LT
        .rx_fifo_rd_en              (rfifo_rd_en),          // Sync to rx_coreclkin
        .rx_is_lockedtoref          (rx_is_lockedtoref),    // Async
        .rx_parallel_data           (rxpcs_din),            // Sync to rx_coreclkin
        .rx_serial_data             (rx_serial),            // 4x10G serial lanes
        .tx_clkout                  (tx_clkout),            // div 33 clk, 156.25 MHz
        .tx_coreclkin               (tx_coreclk_to_pma),    // 312.5 MHz for 40G/156 for AN/322 for LT
        .tx_dll_lock                (tx_dll_lock),          // ??????????????????????
        .tx_fifo_wr_en              (tx_pcs_valid),    // Sync to tx_coreclkin
        .tx_err_ins                 (tx_err_ins), 
        .tx_parallel_data           (tx_pcs_data),          // Sync to tx_coreclkin
        .tx_serial_clk0             ({4{tx_serial_clk}}),   // 5,156.25 MHz
        .tx_serial_data             (tx_serial),            // 4x10G serial lanes
        .tx_analogreset             (tx_analogreset),
        .tx_digitalreset            (tx_digitalreset),
        .rx_analogreset             (rx_analogreset),
        .rx_digitalreset            (rx_digitalreset),
        .tx_cal_busy                (tx_cal_busy),
        .tx_analogreset_stat        (tx_analogreset_stat),
        .tx_digitalreset_stat       (tx_digitalreset_stat),
        .rx_is_lockedtodata         (rx_is_lockedtodata),
        .rx_set_locktoref           (rx_set_locktoref),
        .rx_set_locktodata          (rx_set_locktodata),
        .rx_cal_busy                (rx_cal_busy),
        .rx_analogreset_stat        (rx_analogreset_stat),
        .rx_digitalreset_stat       (rx_digitalreset_stat_pma),
        .rx_data_valid              (rx_data_valid),
        .tx_fifo_full               (tx_fifo_full),
        .tx_fifo_empty              (tx_fifo_empty),
        .tx_fifo_pfull              (tx_fifo_pfull),
        .tx_fifo_pempty             (tx_fifo_pempty),
        .rx_fifo_full               (rx_fifo_full),
        .rx_fifo_empty              (rx_fifo_empty),
        .rx_fifo_pfull              (rx_fifo_pfull),
        .rx_fifo_pempty             (rx_fifo_pempty),
        .reconfig_clk               (reconfig_clk),
        .reconfig_reset             (reconfig_reset),
        .reconfig_write             (reconfig_write_pma),
        .reconfig_read              (reconfig_read_pma),
        .reconfig_address           (reconfig_address_pma),
        .reconfig_writedata         (reconfig_writedata_pma),
        .reconfig_readdata          (reconfig_readdata),
        .reconfig_waitrequest       (reconfig_waitrequest_pma),
        .rx_seriallpbken            (rx_seriallpbken)
    );

assign clk_rx_recover = rx_clkout[1];

// KR4 TRS is needed only for KR4 NOT non-KR4-FEC
generate 
  if (SYNTH_SEQ) begin : KR4_TRS
//===================================================================
//=== Instantiate Reset Sequencer here ==============================
//===================================================================
    alt_e40_kr4_reset_sequencer_top alt_e40_kr4_reset_sequencer_top_inst
    (	
      .clk_in             (reconfig_clk     ), // used for CDC
      .reset              ( pma_reset       ),
      .release_aib_first  (1'b1             ),
      .reset_in_tx        (tx_digitalreset  ),
      .reset_in_rx        (rx_digitalreset  ),
      .start_reset_aib    (start_reset_ctrl_tx_aib),
      .start_reset_pcs    (start_reset_ctrl_tx_pcs),
      .reset_aib_ack      (reset_aib_ack    ),
      .reset_pcs_ack      (reset_pcs_ack    ),
      .rx_transfer_ready  (rx_transfer_ready),
      .tx_transfer_ready  (tx_transfer_ready),
      .rx_aib_reset       (rx_aib_reset     ),
      .rx_pcs_reset       (rx_pcs_reset     ),
      .tx_aib_reset       (tx_aib_reset     ),
      .tx_pcs_reset       (tx_pcs_reset     ),
      .rx_reset_out       (rx_digitalreset_stat_trs_kr),
      .tx_reset_out       (tx_digitalreset_stat)
//===================================================================
//=== End of Reset Sequencer here ==============================
//===================================================================
    );
       assign rx_digitalreset_stat = rx_digitalreset_stat_trs_kr ;  
  end else begin : NO_KR4_TRS
       assign rx_digitalreset_stat = rx_digitalreset_stat_pma;
	   assign rx_aib_reset			= rx_digitalreset;
	   assign tx_aib_reset			= tx_digitalreset;
	   assign rx_pcs_reset			= rx_digitalreset;
	   assign tx_pcs_reset			= tx_digitalreset;
	   assign reset_aib_ack			= 1'b0;
	   assign reset_pcs_ack			= 1'b0;
  end
endgenerate


endmodule

